# file	: startExpr.sh - Start the experiment.
# author: Craig Shallahamer, craig@orapub.com
# last  : 19-Mar-2013

# Purpose: To compare SQL CPU times of Statspack, v$sql, OS, and v$sess_time_model

# As sys do: grant execute on dbms_lock to perfstat;

# Good SP reference: http://www.orafaq.com/wiki/Statspack
# SP levels:
# 0  General Performance Statistics
# 5  Addition Data: SQL Statements
# 6  Addition Data: SQL Plans and SQL Plan Usage
# 7  Addition Data: Segment Level Statistics
# 10 Addition Data: Parent and Child Latches

echo "-------------------------------------"
echo "Setting up clean environment"
echo "-------------------------------------"

load_type=cpu
# for load type cpu it will help cause cbc if a
# larger BC and _db_block_hash_buckets = 256

# set much below the type of sql_scripts to run.
# make sure the script exists with the .sql extention
 #for sql_script in quick


# Make sure to set FOR loops: load, sample_no
# load:   test 2 3
#         real 2 3 4 ... 10
# sample: test 1 2 3
#         real 1 2 3 ... 90

# chill between major exper points
# test=1 real=4
chill_big=4

# testing=3 real test=50 real= 200
sql_exe_num=200

# This is one of the OP Load users, which already contains data.
uid=mg
psd=mg

# Need dba account to turnoff AWR, etc.
dbauid=system
dbapsd=manager

mypid=$$

sqlplus perfstat/perfstat <<EOF
grant select on stats\$sql_summary to $uid ;
exit;
EOF

# DDL
#
sqlplus $uid/$psd <<EOF
drop table op_interim;
create table op_interim (
  cust_id       number,
  cust_status   varchar2(5),
  cust_name     varchar2(200),
  count_orders  number,
  sum_amount    number
);
create or replace function op_get_interval_s(i_intrvl interval day to second) return number is
begin
  return extract(day from i_intrvl) * 86400 + 
         extract(hour from i_intrvl) * 3600 + 
         extract(minute from i_intrvl) * 60 + 
         extract(second from i_intrvl);
  exception
    when others then begin return null; 
  end;
end;
/

create or replace function get_cpu_time_snap_id (snap_id_in number) return number is
begin
declare
  snap_id_v number;
  run_date_v varchar2(100);
  sql_id_v varchar2(50);
  execs_v number;
  cpu_time_v number;

begin

  select snap_id,to_char(t1.last_active_time,'DD HH24:MI:SS'),
         t1.sql_id,t1.executions,t1.cpu_time
  into   snap_id_v, run_date_v, sql_id_v, execs_v, cpu_time_v
  from   stats$sql_summary t1
  where  t1.text_subset like 'begin%declare%sleep_time_v%'
    and  t1.last_active_time = 
        ( select max(t2.last_active_time)
          from stats$sql_summary t2
          where  t2.text_subset like 'begin%declare%sleep_time_v%'
            and  t1.snap_id = t2.snap_id
        )
    and t1.snap_id = snap_id_in;
  
  return cpu_time_v;

  exception
    when others then begin return -9; 
  end;

end;
end;
/
drop table op_sql_results;
create table op_sql_results (
  execs_total number,
  elapsed_s_total number,
  cpu_s_total number,
  wait_all_s_total number,
  wait_nidle_s_total number,
  wait_idle_s_total number,
  avg_time_s_per_exec number
)
/
drop table op_final_results;
create table op_final_results (
  load_type 		varchar2(10),
  sql_script		varchar2(20),
  load			number,
  sample_set_no		number,
  vd_elapsed_s_total	number,
  vd_execs_total	number,
  vd_cpu_systime_s_total	number,
  vd_cpu_sql_s_total	number,
  vd_all_wait_s_total	number,
  vd_nidle_wait_s_total	number,
  vd_idle_wait_s_total	number,
  os_cpu_s_total	number,
  SP_begin_id		number,
  SP_end_id		number
)
/
exit;
EOF


# in the below four sql files:
# - get time, exec sql, get time, store results in op_sql_elaps table
#
#for sql_script in quick threeSec
for sql_script in quick 
do

# Stop all op_load activity  
kp.sh 1
kp.sh 1

sqlplus $dbauid/$dbapsd <<EOF
--disable AWR collection or your numbers can be strangely skewed.
execute dbms_workload_repository.modify_snapshot_settings(interval => 0);
-- disable resource manager to ensure Oracle CPU activity is not throttled down. 
-- If this occurs you will see a quantum related wait event.
alter system set resource_manager_plan = '';
exit;
EOF

# prime the LC with the SQL statement for will be in v$sqlstats
echo ""
echo "Priming the LC with the sql statement."
echo "Expect an error to occur..."
sqlplus -s $uid/$psd @./${sql_script}.sql $sql_exe_num $sql_script
echo ""

# Be happy...
sleep $chill_big

echo "-------------------------------------"
echo "Initial Loading..."
echo "-------------------------------------"

for load in 1   
do
  echo "-------------------------------------"
  echo "  initial loading now... load=$load"
  echo "-------------------------------------"
  opload_go.sh pincursors
  opload_go.sh lio_lite
done
sleep $chill_big

# By load X9, cpu util is easily 100%, because 6 cores
for load in 2 3 4 5 6   
#for load in 2 3 
do
  echo ""
  echo "-------------------------------------"
  echo "LOADING NOW to load=$load"
  echo "-------------------------------------"
  # This type of load will stress the cpu subsystem,
  # specifically LC and CBC memory structures.
  opload_go.sh pincursors
  opload_go.sh lio_lite
  sleep $chill_big

  # get samples for each sql_script, each load, each sample_set_no
  #
  for sample_set_no in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31
  #for sample_set_no in 1 2 3
  do
    echo ""
    echo "   ---------------------------------------------------------------------------------------------"
    echo "   Doing sample sql_script=$sql_script load=$load sample_set_no=$sample_set_no sql_exe_num=$sql_exe_num"
    echo "   ---------------------------------------------------------------------------------------------"

    echo ""
    echo " Gathering Statspack T0 data..."
    echo ""
    sqlplus perfstat/perfstat <<EOF
exec perfstat.statspack.snap(i_snap_level=>5);
exec sys.dbms_lock.sleep($chill_big);
spool /tmp/sp.id
select 'abc123,'||max(snap_id) from stats\$snapshot;
exit;
EOF
    vSpIdT0=`cat /tmp/sp.id | grep 'abc123' | grep -v 'snap_id' | awk -F, '{print $2}'`


    # the capture and store of Oracle related CPU time occurs in the $sql_script
    # exec the sql $sql_exe_num times, store results in op_sql_results 

    vsqlCpuT0=`./getsidcpu.sh $dbauid/$dbapsd | grep 'abc123' | grep -v 'cpu_time' | awk -F, '{print $2}'`

    # The actual sql we are measuring...
    sqlplus -s $uid/$psd @./${sql_script}.sql $sql_exe_num $sql_script

    echo ""
    echo "Sleeping for $chill_big seconds..."
    sleep $chill_big

    vsqlCpuT1=`./getsidcpu.sh $dbauid/$dbapsd | grep 'abc123' | grep -v 'cpu_time' | awk -F, '{print $2}'`
    cpu_sql_s_total=`echo "scale=5;(${vsqlCpuT1}-${vsqlCpuT0})/1000000" | bc`
    echo ""
    echo "         vsqlCpuT0=$vsqlCpuT0 vsqlCpuT1=$vsqlCpuT1 cpu_sql_s_total=$cpu_sql_s_total"

    procCpuT0=`cat /tmp/procCpuT0.txt`
    procCpuT1=`cat /tmp/procCpuT1.txt`
    os_cpu_s_total=`echo "scale=5;${procCpuT1}-${procCpuT0}" | bc`

    echo ""
    echo " Gathering Statspack T1 data..."
    echo ""
    sqlplus perfstat/perfstat <<EOF
exec perfstat.statspack.snap(i_snap_level=>5);
exec sys.dbms_lock.sleep($chill_big);
spool /tmp/sp.id
select 'abc123,'||max(snap_id) from stats\$snapshot;
exit;
EOF
    vSpIdT1=`cat /tmp/sp.id | grep 'abc123' | grep -v 'snap_id' | awk -F, '{print $2}'`

    sqlplus $uid/$psd <<EOF
select * from op_sql_results
/
insert into op_final_results  
select '${load_type}', '${sql_script}' , ${load} , ${sample_set_no} ,
       elapsed_s_total, execs_total, cpu_s_total, ${cpu_sql_s_total}, 
       wait_all_s_total, wait_nidle_s_total, wait_idle_s_total,
       ${os_cpu_s_total},${vSpIdT0},${vSpIdT1}
from   op_sql_results
/
exit;
EOF
    sleep $chill_big
  done 
done

done # for sql_stmt in ./quick.sql ...

echo "-------------------------------------"
echo "-------------------------------------"
echo " Stopping Load"
echo "-------------------------------------"
echo "-------------------------------------"
kp.sh 1
kp.sh 1

sqlplus $uid/$psd <<EOF
set tab off
col sql_script format a10
set linesize 200
select * from op_final_results order by 1,2,3,4;

select LOAD_TYPE,SQL_SCRIPT,LOAD,SAMPLE_SET_NO,
       vd_execs_total,
       vd_elapsed_s_TOTAL,
       vd_cpu_systime_s_total, vd_cpu_sql_s_total, 
       os_cpu_s_total,
       vd_elapsed_s_total/vd_execs_total avg_el_time,
       sp_begin_id, sp_end_id,
       (get_cpu_time_snap_id(sp_end_id)-get_cpu_time_snap_id(sp_begin_id))/1000000 sp_cpu_s
from   op_final_results
order by 1,2,3,4;

select LOAD_TYPE,SQL_SCRIPT,LOAD,SAMPLE_SET_NO,
       vd_execs_total,
       vd_elapsed_s_TOTAL,
       vd_cpu_systime_s_total, vd_cpu_sql_s_total, 
       os_cpu_s_total,
       vd_elapsed_s_total/vd_execs_total avg_el_time,
       sp_begin_id, sp_end_id
from   op_final_results
order by 1,2,3,4;
exit;
EOF

